﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2017 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Data.Linq;
using System.Linq;

namespace HeuristicLab.Services.Hive.DataAccess.Daos {
  public class ProjectDao : GenericDao<Guid, Project> {
    public ProjectDao(DataContext dataContext) : base(dataContext) { }

    public override Project GetById(Guid id) {
      return GetByIdQuery(DataContext, id);
    }

    public IEnumerable<Project> GetChildProjectsById(Guid id) {
      return DataContext.ExecuteQuery<Project>(GetChildProjectsByIdQuery, id);
    }

    public IEnumerable<Guid> GetChildProjectIdsById(Guid id) {
      return DataContext.ExecuteQuery<Guid>(GetChildProjectIdsByIdQuery, id);
    }

    public IEnumerable<Project> GetParentProjectsById(Guid id) {
      return DataContext.ExecuteQuery<Project>(GetParentProjectsByIdQuery, id);
    }

    public IEnumerable<Guid> GetParentProjectIdsById(Guid id) {
      return DataContext.ExecuteQuery<Guid>(GetParentProjectIdsByIdQuery, id);
    }

    public IEnumerable<Project> GetCurrentAndParentProjectsById(Guid id) {
      return DataContext.ExecuteQuery<Project>(GetCurrentAndParentProjectsByIdQuery, id);
    }

    public IEnumerable<Guid> GetCurrentAndParentProjectIdsById(Guid id) {
      return DataContext.ExecuteQuery<Guid>(GetCurrentAndParentProjectIdsByIdQuery, id);
    }

    #region Compiled queries
    private static readonly Func<DataContext, Guid, Project> GetByIdQuery =
      CompiledQuery.Compile((DataContext db, Guid projectId) =>
        (from project in db.GetTable<Project>()
         where project.ProjectId == projectId
         select project).SingleOrDefault());
    #endregion

    #region String queries
    private const string GetChildProjectsByIdQuery = @"
    	WITH ptree AS
      (
	      SELECT ProjectId, ParentProjectId
	      FROM [Project]
	      UNION ALL
	      SELECT pt.ProjectId, p.ParentProjectId
	      FROM [Project] p
	      JOIN ptree pt ON pt.ParentProjectId = p.ProjectId AND p.ParentProjectId <> p.ProjectId AND pt.ParentProjectId <> pt.ProjectId
      )
      SELECT DISTINCT pro.*
      FROM ptree, [Project] pro
      WHERE ptree.ParentProjectId = {0}
	    AND ptree.ProjectId = pro.ProjectId
    ";
    private const string GetChildProjectIdsByIdQuery = @"
      WITH ptree AS
      (
	      SELECT ProjectId, ParentProjectId
	      FROM [Project]
	      UNION ALL
	      SELECT pt.ProjectId, r.ParentProjectId
	      FROM [Project] r
	      JOIN ptree pt ON pt.ParentProjectId = r.ProjectId AND r.ParentProjectId <> r.ProjectId AND pt.ParentProjectId <> pt.ProjectId
      )
      SELECT DISTINCT ptree.ProjectId
      FROM ptree
      WHERE ptree.ParentProjectId = {0}
    ";
    private const string GetParentProjectsByIdQuery = @"
      WITH pbranch AS
      (
	      SELECT ProjectId, ParentProjectId
        FROM [Project]
	      UNION ALL
	      SELECT pb.ProjectId, p.ParentProjectId
	      FROM [Project] p
	      JOIN pbranch pb ON pb.ParentProjectId = p.ProjectId AND p.ParentProjectId <> p.ProjectId AND pb.ParentProjectId <> pb.ProjectId
      )
      SELECT DISTINCT pro.*
      FROM pbranch, [Project] pro
      WHERE pbranch.ProjectId = {0}
      AND pbranch.ParentProjectId = pro.ProjectId
    ";
    private const string GetParentProjectIdsByIdQuery = @"
      WITH pbranch AS
      (
	      SELECT ProjectId, ParentProjectId
        FROM [Project]
	      UNION ALL
	      SELECT pb.ProjectId, p.ParentProjectId
	      FROM [Project] p
	      JOIN pbranch pb ON pb.ParentProjectId = p.ProjectId AND p.ParentProjectId <> p.ProjectId AND pb.ParentProjectId <> pb.ProjectId
      )
      SELECT DISTINCT pbranch.ParentProjectId
      FROM pbranch
      WHERE pbranch.ProjectId = {0}
    ";
    private const string GetCurrentAndParentProjectsByIdQuery = @"
      WITH pbranch AS
      (
	      SELECT ProjectId, ParentProjectId
	      FROM Project
	      WHERE ProjectId = {0}
	      UNION ALL
	      SELECT p.ProjectId, p.ParentProjectId
	      FROM Project p
	      JOIN pbranch pb ON pb.ParentProjectId = p.ProjectId
      )
      SELECT DISTINCT pro.*
      FROM pbranch, Project pro
      WHERE pbranch.ProjectId = pro.ProjectId
    ";
    private const string GetCurrentAndParentProjectIdsByIdQuery = @"
      WITH pbranch AS
      (
	      SELECT ProjectId, ParentProjectId
	      FROM Project
	      WHERE ProjectId = {0}
	      UNION ALL
	      SELECT p.ProjectId, p.ParentProjectId
	      FROM Project p
	      JOIN pbranch pb ON pb.ParentProjectId = p.ProjectId
      )
      SELECT DISTINCT pbranch.ProjectId
      FROM pbranch
    ";
    #endregion
  }
}
